@@ -1,11 +1,11 @@
 /*
  * dict.c	Routines to read the dictionary file.
  *
- * Version:	$Id: dict.c,v 1.30.4.1 2002/11/29 16:46:37 aland Exp $
+ * Version:	$Id: dict.c,v 1.39 2003/02/19 15:10:20 cmiller Exp $
  *
  */
 
-static const char rcsid[] = "$Id: dict.c,v 1.30.4.1 2002/11/29 16:46:37 aland Exp $";
+static const char rcsid[] = "$Id: dict.c,v 1.39 2003/02/19 15:10:20 cmiller Exp $";
 
 #include	"autoconf.h"
 
@@ -24,22 +24,16 @@ static DICT_ATTR	*dictionary_attributes = NULL;
 static DICT_VALUE	*dictionary_values = NULL;
 static DICT_VENDOR	*dictionary_vendors = NULL;
 
-static const char *dtypes[] = {
-	"string",
-	"integer",
-	"ipaddr",
-	"date",
-	"abinary",
-	"octets",
-	NULL,
+static const LRAD_NAME_NUMBER type_table[] = {
+	{ "string",	PW_TYPE_STRING },
+	{ "integer",	PW_TYPE_INTEGER },
+	{ "ipaddr",	PW_TYPE_IPADDR },
+	{ "date",	PW_TYPE_DATE },
+	{ "abinary",	PW_TYPE_ABINARY },
+	{ "octets",	PW_TYPE_OCTETS },
+	{ NULL, 0 }
 };
 
-#ifdef WITH_DICT_NOCASE
-#define DICT_STRCMP strcasecmp
-#else
-#define DICT_STRCMP strcmp
-#endif
-
 /*
  *	Quick pointers to the base 0..255 attributes.
  *
@@ -179,6 +173,7 @@ int dict_addattr(const char *name, int vendor, int type, int value, ATTR_FLAGS f
 
 int dict_addvalue(const char *namestr, char *attrstr, int value)
 {
+	DICT_ATTR	*dattr;
 	DICT_VALUE	*dval;
 
 	if (strlen(namestr) > (sizeof(dval->name) -1)) {
@@ -195,19 +190,232 @@ int dict_addvalue(const char *namestr, char *attrstr, int value)
 		librad_log("dict_addvalue: out of memory");
 		return -1;
 	}
-	
+
 	strcpy(dval->name, namestr);
 	strcpy(dval->attrname, attrstr);
-	dval->attr = 0;		/* ??? */
+
+	/*
+	 *	Remember which attribute is associated with this
+	 *	value, if possible.
+	 */
+	dattr = dict_attrbyname(attrstr);
+	if (dattr) {
+		dval->attr = dattr->attr;
+	} else {
+		dval->attr = 0;
+	}
 	dval->value = value;
 	
 	/* Insert at front. */
 	dval->next = dictionary_values;
 	dictionary_values = dval;
-			
+
 	return 0;
 }
 
+/*
+ *	Process the ATTRIBUTE command
+ */
+static int process_attribute(const char* fn, const int line, const int block_vendor, const char* data) {
+	int		vendor;
+	char		namestr[256];
+	char		valstr[256];
+	char		typestr[256];
+	char		optstr[256];
+	int		value;
+	int		type;
+	char		*s, *c;
+	ATTR_FLAGS	flags;
+
+	vendor = 0;
+	optstr[0] = 0;
+	if(sscanf(data, "%s%s%s%s", namestr, valstr, typestr, optstr) < 3) {
+		librad_log("dict_init: %s[%d]: invalid ATTRIBUTE line",
+			fn, line);
+		return -1;
+	}
+
+	/*
+	 *	Validate all entries
+	 */
+	if (!isdigit((int) *valstr)) {
+		librad_log("dict_init: %s[%d]: invalid value", fn, line);
+		return -1;
+	}
+	if (valstr[0] != '0')
+		value = atoi(valstr);
+	else
+		sscanf(valstr, "%i", &value);
+
+	/*
+	 *	find the type of the attribute.
+	 */
+	type = lrad_str2int(type_table, typestr, -1);
+	if (type < 0) {
+		librad_log("dict_init: %s[%d]: invalid type \"%s\"",
+			fn, line, typestr);
+		return -1;
+	}
+
+	/*
+	 *	Ignore comments
+	 */
+	if (optstr[0] == '#') optstr[0] = '\0';
+
+	/*
+	 *	Only look up the vendor if the string
+	 *	is non-empty.
+	 */
+
+	memset(&flags, 0, sizeof(flags));
+	s = strtok(optstr, ",");
+	while(s) {
+		if (strcmp(s, "has_tag") == 0 ||
+		    strcmp(s, "has_tag=1") == 0) {
+			 /* Boolean flag, means this is a
+			    tagged attribute */
+			 flags.has_tag = 1;
+		}
+		else if (strncmp(s, "len+=", 5) == 0 ||
+			 strncmp(s, "len-=", 5) == 0) { 
+			  /* Length difference, to accomodate
+			     braindead NASes & their vendors */
+			  flags.len_disp = strtol(s + 5, &c, 0);
+			  if (*c) {
+				librad_log("dict_init: %s[%d] invalid option %s",
+					   fn, line, s);
+				return -1;
+			  }
+			  if (s[3] == '-') {
+				flags.len_disp = -flags.len_disp;
+			  }
+		}
+		else if (strncmp(s, "encrypt=", 8) == 0) {
+			  /* Encryption method, defaults to 0 (none).
+			     Currently valid is just type 2,
+			     Tunnel-Password style, which can only
+			     be applied to strings. */    
+			  flags.encrypt = strtol(s + 8, &c, 0);
+			  if (*c) {
+				librad_log( "dict_init: %s[%d] invalid option %s",
+					   fn, line, s);
+				return -1;
+			  }
+		}
+		else {
+			  /* Must be a vendor 'flag'... */
+			  if (strncmp(s, "vendor=", 5) == 0) {
+				/* New format */
+				s += 5;   
+			  }
+	 
+			  vendor = dict_vendorname(s);
+			  if (!vendor) {
+				librad_log( "dict_init: %s[%d]: unknown vendor %s",
+					   fn, line, optstr);
+				return -1;
+			  }
+			  if (block_vendor && optstr[0] &&
+			      (block_vendor != vendor)) {
+				librad_log("dict_init: %s[%d]: mismatched vendor %s within BEGIN-VENDOR/END-VENDOR block",
+					   fn, line, optstr);
+				return -1;
+			  }
+		}
+		s = strtok(NULL, ",");
+	}
+
+	if (block_vendor) vendor = block_vendor;
+
+	if (dict_addattr(namestr, vendor, type, value, flags) < 0) {
+		librad_log("dict_init: %s[%d]: %s",
+			   fn, line, librad_errstr);
+		return -1;
+	}
+
+	return 0;
+}
+
+
+/*
+ *	Process the VALUE command
+ */
+static int process_value(const char* fn, const int line, const int block_vendor, const char* data) {
+	char	namestr[256];
+	char	valstr[256];
+	char	attrstr[256];
+	int	value;
+
+	if (sscanf(data, "%s%s%s", attrstr, namestr, valstr) != 3) {
+		librad_log("dict_init: %s[%d]: invalid VALUE line",
+			fn, line);
+		return -1;
+	}
+	/*
+	 *	For Compatibility, skip "Server-Config"
+	 */
+	if (strcasecmp(attrstr, "Server-Config") == 0)
+		return 0;
+
+	/*
+	 *	Validate all entries
+	 */
+	if (!isdigit((int) *valstr)) {
+		librad_log("dict_init: %s[%d]: invalid value",
+			fn, line);
+		return -1;
+	}
+	if (valstr[0] != '0')
+		value = atoi(valstr);
+	else
+		sscanf(valstr, "%i", &value);
+
+	if (dict_addvalue(namestr, attrstr, value) < 0) {
+		librad_log("dict_init: %s[%d]: %s", 
+			   fn, line, librad_errstr);
+		return -1;
+	}
+
+	return 0;
+}
+
+
+/*
+ *	Process the VENDOR command
+ */
+static int process_vendor(const char* fn, const int line, const int block_vendor, const char* data) {
+	char	valstr[256];
+	char	attrstr[256];
+	int	value;
+
+	if (sscanf(data, "%s%s", attrstr, valstr) != 2) {
+		librad_log(
+		"dict_init: %s[%d] invalid VENDOR entry",
+			fn, line);
+		return -1;
+	}
+
+	/*
+	 *	 Validate all entries
+	 */
+	if (!isdigit((int) *valstr)) {
+		librad_log("dict_init: %s[%d]: invalid value",
+			fn, line);
+		return -1;
+	}
+	value = atoi(valstr);
+
+	/* Create a new VENDOR entry for the list */
+	if (dict_addvendor(attrstr, value) < 0) {
+		librad_log("dict_init: %s[%d]: %s",
+			   fn, line, librad_errstr);
+		return -1;
+	}
+
+	return 0;
+}
+
+
 /*
  *	Initialize the dictionary.
  */
@@ -216,23 +424,13 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 	FILE	*fp;
 	char 	dirtmp[256];
 	char	buf[256];
-	char	namestr[256];
-	char	valstr[256];
-	char	attrstr[256];
-	char	typestr[256];
 	char	optstr[256];
-	char	*p, *s, *c;
+	char	*p;
 	char	*keyword;
 	char	*data;
 	int	line = 0;
-	int	value;
-	int	type;
 	int	vendor;
 	int	block_vendor;
-	int	is_attrib;
-	int	vendor_usr_seen = 0;
-	int	is_nmc;
-	ATTR_FLAGS  flags;
 
 	if (strlen(fn) >= sizeof(dirtmp) / 2 ||
 	    strlen(dir) >= sizeof(dirtmp) / 2) {
@@ -267,27 +465,27 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 	block_vendor = 0;
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
-
 		line++;
 		if (buf[0] == '#' || buf[0] == 0 ||
 		    buf[0] == '\n' || buf[0] == '\r')
 			continue;
 
 		/*
-		 *  Comments should NOT be appearing anywhere but
-		 *  as comments;
+		 *  Comment characters should NOT be appearing anywhere but
+		 *  as start of a comment;
 		 */
 		p = strchr(buf, '#');
 		if (p) *p = '\0';
 
 		keyword = strtok(buf, " \t\r\n");
-		if (keyword == NULL)
+		if (keyword == NULL) {
 			continue;
+		}
 
 		data    = strtok(NULL, "\r\n");
 		if (data == NULL || data[0] == 0) {
-			librad_log("dict_init: %s[%d]: invalid entry",
-				fn, line);
+			librad_log("dict_init: %s[%d]: invalid entry for keyword %s",
+				fn, line, keyword);
 			fclose(fp);
 			return -1;
 		}
@@ -307,254 +505,38 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 		/*
 		 *	Perhaps this is an attribute.
 		 */
-		is_attrib = 0;
-		if (DICT_STRCMP(keyword, "ATTRIBUTE") == 0)
-			is_attrib = 1;
-
-		is_nmc = 0;
-		if (DICT_STRCMP(keyword, "ATTRIB_NMC") == 0)
-			is_attrib = is_nmc = 1;
-
-		if (is_attrib) {
-
-			vendor = 0;
-			optstr[0] = 0;
-			if(sscanf(data, "%s%s%s%s", namestr,
-					valstr, typestr, optstr) < 3) {
-				librad_log(
-					"dict_init: %s[%d]: invalid ATTRIBUTE line",
-					fn, line);
-				fclose(fp);
-				return -1;
-			}
-
-			/*
-			 *	Convert ATTRIB_NMC into our format.
-			 *	We might need to add USR to the list of
-			 *	vendors first.
-			 */
-			if (is_nmc && optstr[0] == 0) {
-				if (!vendor_usr_seen) {
-					if (dict_addvendor("USR", VENDORPEC_USR) < 0) {
-						librad_log("dict_init: %s[%d]: %s", fn, line, librad_errstr);
-						fclose(fp);
-						return -1;
-					}
-					vendor_usr_seen = 1;
-				}
-				strcpy(optstr, "USR");
-			}
-
-			/*
-			 *	Validate all entries
-			 */
-			if (!isdigit((int) *valstr)) {
-				librad_log("dict_init: %s[%d]: invalid value",
-					fn, line);
-				fclose(fp);
-				return -1;
-			}
-			if (valstr[0] != '0')
-				value = atoi(valstr);
-			else
-				sscanf(valstr, "%i", &value);
-
-			/*
-			 *	find the type.
-			 */
-			for (type = 0; dtypes[type]; type++) {
-				if (DICT_STRCMP(typestr, dtypes[type]) == 0)
-					break;
-			}
-			if (dtypes[type] == NULL) {
-				librad_log("dict_init: %s[%d]: invalid type",
-					fn, line);
-				fclose(fp);
-				return -1;
-			}
-
-			/*
-			 *	Ignore comments
-			 */
-			if (optstr[0] == '#') {
-				optstr[0] = '\0';
-			}
-
-			/*
-			 *	Only look up the vendor if the string
-			 *	is non-empty.
-			 */
-
-			memset(&flags, 0, sizeof(flags));
-			s = strtok(optstr, ",");
-			while(s) {
-			        if (strcmp(s, "has_tag") == 0 ||
-				    strcmp(s, "has_tag=1") == 0) {
-				         /* Boolean flag, means this is a
-					    tagged attribute */
-				         flags.has_tag = 1;
-				}
-				else if (strncmp(s, "len+=", 5) == 0 ||
-					 strncmp(s, "len-=", 5) == 0) { 
-				          /* Length difference, to accomodate
-					     braindead NASes & their vendors */
-				          flags.len_disp = strtol(s + 5, &c, 0);
-					  if (*c) {
-					        librad_log(
-							   "dict_init: %s[%d] invalid option %s",
-							   fn, line, s);
-						fclose(fp);
-						return -1;
-					  }
-					  if (s[3] == '-') {
-					        flags.len_disp = -flags.len_disp;
-					  }
-				}
-				else if (strncmp(s, "encrypt=", 8) == 0) {
-				          /* Encryption method, defaults to 0 (none).
-					     Currently valid is just type 2,
-					     Tunnel-Password style, which can only
-					     be applied to strings. */    
-				          flags.encrypt = strtol(s + 8, &c, 0);
-					  if (*c) {
-					        librad_log(
-							   "dict_init: %s[%d] invalid option %s",
-							   fn, line, s);
-						fclose(fp);
-						return -1;
-					  }
-				}
-				else {
-				          /* Must be a vendor 'flag'... */
-				          if (strncmp(s, "vendor=", 5) == 0) {
-					        /* New format */
-					        s += 5;   
-					  }
-                         
-					  vendor = dict_vendorname(s);
-					  if (!vendor) {
-					        librad_log(
-							   "dict_init: %s[%d]: unknown vendor %s",
-							   fn, line, optstr);
-						fclose(fp);
-						return -1;
-					  }
-					  if (block_vendor && optstr[0] &&
-					      (block_vendor != vendor)) {
-					        librad_log(
-							   "dict_init: %s[%d]: mismatched vendor %s within BEGIN-VENDOR/END-VENDOR block",
-							   fn, line, optstr);
-						fclose(fp);
-						return -1;
-					  }
-				}
-				s = strtok(NULL, ",");
-			}
- 
-			if (block_vendor) vendor = block_vendor;
-
-			if (dict_addattr(namestr, vendor, type, value, flags) < 0) {
-				librad_log("dict_init: %s[%d]: %s",
-					   fn, line, librad_errstr);
+		if (strcasecmp(keyword, "ATTRIBUTE") == 0) {
+			if (process_attribute(fn, line, block_vendor, data) == -1) {
 				fclose(fp);
 				return -1;
 			}
 			continue;
-		} /* ATTRIBUTE, or ATTRIB_NMC */
+		}
 
 		/*
 		 *	Process VALUE lines.
 		 */
-		if (DICT_STRCMP(keyword, "VALUE") == 0) {
-			if (sscanf(data, "%s%s%s", attrstr,
-						namestr, valstr) != 3) {
-				librad_log("dict_init: %s[%d]: invalid VALUE line",
-					fn, line);
-				fclose(fp);
-				return -1;
-			}
-			/*
-			 *	For Compatibility, skip "Server-Config"
-			 */
-			if (DICT_STRCMP(attrstr, "Server-Config") == 0)
-				continue;
-
-			/*
-			 *	Validate all entries
-			 */
-			if (!isdigit((int) *valstr)) {
-				librad_log("dict_init: %s[%d]: invalid value",
-					fn, line);
-				fclose(fp);
-				return -1;
-			}
-			if (valstr[0] != '0')
-				value = atoi(valstr);
-			else
-				sscanf(valstr, "%i", &value);
-
-#if 0
-			/*
-			 *	This WOULD be nice, but the dictionary
-			 *	files require heavy editing to enable it,
-			 *	as many entries are out of order. :(
-			 */
-			if (dict_attrbyname(attrstr) == NULL) {
-				librad_log("dict_init: %s[%d]: No previously defined ATTRIBUTE %s for VALUE", 
-					   fn, line, attrstr);
-				fclose(fp);
-				return -1;
-			}
-#endif
-
-			if (dict_addvalue(namestr, attrstr, value) < 0) {
-				librad_log("dict_init: %s[%d]: %s", 
-					   fn, line, librad_errstr);
+		if (strcasecmp(keyword, "VALUE") == 0) {
+			if (process_value(fn, line, block_vendor, data) == -1) {
 				fclose(fp);
 				return -1;
 			}
 			continue;
-		} /* VALUE */
+		}
+
 
 		/*
 		 *	Process VENDOR lines.
 		 */
-		if (DICT_STRCMP(keyword, "VENDOR") == 0) {
-
-			if (sscanf(data, "%s%s", attrstr, valstr) != 2) {
-				librad_log(
-				"dict_init: %s[%d] invalid VENDOR entry",
-					fn, line);
+		if (strcasecmp(keyword, "VENDOR") == 0) {
+			if (process_vendor(fn, line, block_vendor, data) == -1) {
 				fclose(fp);
 				return -1;
 			}
-
-			/*
-			 *	 Validate all entries
-			 */
-			if (!isdigit((int) *valstr)) {
-				librad_log("dict_init: %s[%d]: invalid value",
-					fn, line);
-				fclose(fp);
-				return -1;
-			}
-			value = atoi(valstr);
-
-			/* Create a new VENDOR entry for the list */
-			if (dict_addvendor(attrstr, value) < 0) {
-				librad_log("dict_init: %s[%d]: %s",
-					   fn, line, librad_errstr);
-				fclose(fp);
-				return -1;
-			}
-
-			if (value == VENDORPEC_USR)
-				vendor_usr_seen = 1;
-
 			continue;
-		} /* VENDOR */
+		}
 
-		if (DICT_STRCMP(keyword, "BEGIN-VENDOR") == 0) {
+		if (strcasecmp(keyword, "BEGIN-VENDOR") == 0) {
 			optstr[0] = 0;
 			if (sscanf(data, "%s", optstr) != 1) {
 				librad_log(
@@ -576,7 +558,7 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 			continue;
 		} /* BEGIN-VENDOR */
 
-		if (DICT_STRCMP(keyword, "END-VENDOR") == 0) {
+		if (strcasecmp(keyword, "END-VENDOR") == 0) {
 			optstr[0] = 0;
 			if (sscanf(data, "%s", optstr) != 1) {
 				librad_log(
@@ -678,7 +660,7 @@ DICT_ATTR * dict_attrbyname(const char *name)
 	DICT_ATTR	*a;
 
 	for (a = dictionary_attributes; a; a = a->next) {
-		if (DICT_STRCMP(a->name, name) == 0)
+		if (strcasecmp(a->name, name) == 0)
 			return a;
 	}
 
@@ -713,7 +695,7 @@ DICT_VALUE * dict_valbyname(int attr, const char *name)
 
 	for (v = dictionary_values; v; v = v->next) {
 		if ((attr == 0 || v->attr == attr) &&
-		    DICT_STRCMP(v->name, name) == 0)
+		    strcasecmp(v->name, name) == 0)
 		 return v;
                
 	}
@@ -732,7 +714,7 @@ int dict_vendorname(const char *name)
 	 *	Find the vendor, if any.
 	 */
 	for (v = dictionary_vendors; v; v = v->next) {
-		if (DICT_STRCMP(name, v->vendorname) == 0) {
+		if (strcasecmp(name, v->vendorname) == 0) {
 			return v->vendorpec;
 		}
 	}
