@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.288.2.1 2003/07/16 14:42:50 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.288.2.3 2003/08/26 12:32:13 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.288.2.1 2003/07/16 14:42:50 aland Exp $";
+"$Id: radiusd.c,v 1.288.2.3 2003/08/26 12:32:13 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -835,6 +835,13 @@ int main(int argc, char *argv[])
 		}
 
 		if (need_reload) {
+#if HAVE_PTHREAD_H
+			/*
+			 *	Threads: wait for all threads to stop
+			 *	processing before re-loading the
+			 *	config, so we don't pull the rug out
+			 *	from under them.
+			 */
 		        int max_wait = 0;
 		        for(;;) {
 			        /*
@@ -850,6 +857,7 @@ int main(int argc, char *argv[])
 				sleep(1);
 				max_wait++;
 			}
+#endif
 			if (read_mainconfig(TRUE) < 0) {
 				exit(1);
 			}
