@@ -26,7 +26,7 @@
  *
  */
 char radrelay_rcsid[] =
-"$Id: radrelay.c,v 1.8 2002/10/30 20:38:18 aland Exp $";
+"$Id: radrelay.c,v 1.13.2.2 2003/08/31 14:01:19 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -60,20 +60,13 @@ const char *progname;
 int debug_flag = 0;
 const char *radlog_dir = NULL;
 radlog_dest_t radlog_dest = RADLOG_FILES;
-const char *radutmp_file = NULL;
 
-int proxy_synchronous = TRUE;
-int proxy_fallback = FALSE;
 const char *radius_dir = NULL;
 const char *radacct_dir = NULL;
 const char *radlib_dir = NULL;
 int auth_port = 0;
 int acct_port;
 uint32_t myip = INADDR_ANY;
-int proxy_retry_delay = RETRY_DELAY;
-int proxy_retry_count = RETRY_COUNT;
-int proxy_dead_time;
-int max_proxies = MAX_PROXIES;
 int log_stripped_names;
 struct main_config_t mainconfig;
 
@@ -123,12 +116,13 @@ char *c_secret = NULL;
 char *c_shortname = NULL;
 
 struct relay_request slots[NR_SLOTS];
+char id_map[256];
 int request_head;
-int radius_id;
 int got_sigterm = 0;
 int debug = 0;
 
 
+int get_radius_id(void);
 void sigterm_handler(int sig);
 void ms_sleep(int msec);
 int isdateline(char *d);
@@ -141,6 +135,30 @@ int find_shortname(char *shortname, char **host, char **secret);
 void usage(void);
 
 
+/*
+ * Get a radius id which is not
+ * currently being used (outstanding request)
+ * Since NR_SLOTS < 256 we can't
+ * have more outstanding requests than radius ids
+ */
+int get_radius_id()
+{
+	unsigned int id = 0;
+
+	for(id = 0; id < 256; id++){
+		if (id_map[id] == 0)
+			break;
+	}
+	if (id == 256 || id_map[id] != 0){
+		fprintf(stdout, "get_radius_id(): No IDs available. Something is very wrong\n");
+		return -1;
+	}
+	id_map[id] = 1;
+	fprintf(stdout, "get_radius_id(): Assign RADIUS ID = %d\n",id);
+
+	return id;
+}
+
 void sigterm_handler(int sig)
 {
 	signal(sig, sigterm_handler);
@@ -191,6 +209,7 @@ int read_one(FILE *fp, struct relay_request *r_req)
 	int skip;
 	long fpos;
 	int x;
+	unsigned int i = 0;
 
 	/* Never happens */
 	if (r_req->state == STATE_FULL)
@@ -208,10 +227,13 @@ int read_one(FILE *fp, struct relay_request *r_req)
 	fpos = ftell(fp);
 	fseek(fp, 0L, SEEK_SET);
 	do {
-		x = rad_lockfd(fileno(fp), 0);
+		x = rad_lockfd_nonblock(fileno(fp), 0);
 		if (x == -1)
 			ms_sleep(25);
-	} while (x == -1);
+	} while (x == -1 && i++ < 80);
+
+	if (x == -1)
+		return 0;
 
 	s = NULL;
 	fseek(fp, fpos, SEEK_SET);
@@ -226,7 +248,8 @@ int read_one(FILE *fp, struct relay_request *r_req)
 		 * around.
 		 */
 		if (buf[strlen(buf) - 1] != '\n') {
-			fseek(fp, fpos, SEEK_SET);
+			fprintf(stdout, "read_one: BROKEN ATTRIBUTE\n");
+			fseek(fp, fpos + strlen(buf), SEEK_SET);
 			break;
 		}
 		if (r_req->state == STATE_BUSY1) {
@@ -281,8 +304,7 @@ int read_one(FILE *fp, struct relay_request *r_req)
 			r_req->timestamp -= vp->lvalue;
 			vp->lvalue = 0;
 		}
-		r_req->req->id = radius_id & 0xff;
-		radius_id++;
+		r_req->req->id = get_radius_id();
 	}
 
 	if (s == NULL) {
@@ -349,6 +371,8 @@ int do_recv(struct relay_misc *r_args)
 			 *	Got it. Clear slot.
 			 *	FIXME: check reponse digest ?
 			 */
+			id_map[r->req->id] = 0;
+			fprintf(stdout, "do_recv: Free RADIUS ID = %d\n",r->req->id);
 			if (r->req->vps != NULL) {
 				pairfree(&r->req->vps);
 				r->req->vps = NULL;
@@ -382,6 +406,9 @@ int do_send(struct relay_request *r, char *secret)
 	 *	Prevent loops.
 	 */
 	if (r->client_ip == r->req->dst_ipaddr) {
+		fprintf(stdout, "do_send: Client-IP == Dest-IP. Droping packet.\n");
+		fprintf(stdout, "do_send: Free RADIUS ID = %d\n",r->req->id);
+		id_map[r->req->id] = 0;
 		if (r->req->vps != NULL) {
 			pairfree(&r->req->vps);
 			r->req->vps = NULL;
@@ -403,6 +430,23 @@ int do_send(struct relay_request *r, char *secret)
 	now = time(NULL);
 	if (r->retrans > now)
 		return 0;
+	/*
+	 * If we are resending a packet we *need* to
+	 * change the radius packet id since the request
+	 * authenticator is different (due to different
+	 * Acct-Delay-Time value).
+	 * Otherwise the radius server may consider the
+	 * packet a duplicate and we 'll get caught in a
+	 * loop. 
+	 */
+	if (r->retrans > 0){
+		id_map[r->req->id] = 0;
+		r->req->id = get_radius_id();
+		if (r->req->data != NULL){
+			free(r->req->data);
+			r->req->data = NULL;
+		}
+	}
 	r->retrans = now + 3;
 
 	/*
@@ -600,10 +644,11 @@ void loop(struct relay_misc *r_args)
 		/*
 		 *	See if there's anything to send.
 		 */
+		n=0;
 		for (i = 0; i < NR_SLOTS; i++) {
 			if (slots[i].state == STATE_FULL) {
 				n += do_send(&slots[i], r_args->secret);
-				if (n > 1) ms_sleep(25);
+				ms_sleep(140);
 				if (n > NR_SLOTS / 2)
 					break;
 			}
@@ -687,6 +732,7 @@ int main(int argc, char **argv)
 	char *shortname;
 	char *p;
 	int c;
+	int i;
 	int dontfork = 0;
 	struct relay_misc r_args;
 	FILE *sfile_fp;
@@ -912,6 +958,12 @@ int main(int argc, char **argv)
 #endif
 	}
 
+	/*
+	 * Initialize the radius id map
+	 */
+	for(i=0;i<256;i++)
+		id_map[i] = 0;
+
 	/*
 	 *	Call main processing loop.
 	 */
