@@ -1,7 +1,7 @@
 /*
  * radzap.c	Zap a user from the radutmp and radwtmp file.
  *
- * Version:	$Id: radzap.c,v 1.34 2002/10/30 20:38:18 aland Exp $
+ * Version:	$Id: radzap.c,v 1.37 2003/03/18 05:50:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -48,18 +48,11 @@ const char *radlog_dir = NULL;
 const char *radius_dir = NULL;
 const char *radacct_dir = NULL;
 const char *radlib_dir = NULL;
+radlog_dest_t radlog_dest = RADLOG_FILES;
 int debug_flag = 0;
-int proxy_synchronous = TRUE;
-int proxy_fallback = FALSE;
 int auth_port = 0;
 int acct_port;
-int proxy_retry_delay = RETRY_DELAY;
-int proxy_retry_count = RETRY_COUNT;
-int proxy_dead_time;
-int max_proxies = MAX_PROXIES;
 int log_stripped_names;
-radlog_dest_t radlog_dest = RADLOG_FILES;
-const char *radutmp_file = NULL;
 struct main_config_t mainconfig;
 uint32_t radiusip = INADDR_NONE;
 static void usage(void);
@@ -80,7 +73,7 @@ static int radutmp_lookup(struct radutmp *u, uint32_t nasaddr,
 {
 	int fd;
 
-	if ((fd = open(radutmp_file, O_RDONLY|O_CREAT, 0644)) >= 0) {
+	if ((fd = open(radutmpconfig.radutmp_fn, O_RDONLY|O_CREAT, 0644)) >= 0) {
 		/*
 		 *	Lock the utmp file.
 		 */
@@ -132,7 +125,7 @@ static void usage(void)
  */
 int main(int argc, char **argv)
 {
-	CONF_SECTION *maincs, *cs;
+	CONF_SECTION *cs;
 	NAS *nas;
 	uint32_t ip = 0;
 	uint32_t nas_port = ~0;
@@ -207,13 +200,13 @@ int main(int argc, char **argv)
 	memset(&mainconfig, 0, sizeof(mainconfig));
 
         /* Read radiusd.conf */
-	if ((maincs = read_radius_conf_file()) == NULL) {
+	if (read_mainconfig(0) < 0) {
 		fprintf(stderr, "%s: Error reading radiusd.conf.\n", argv[0]);
 		exit(1);
 	}
 
         /* Read the radutmp section of radiusd.conf */
-        cs = cf_section_sub_find(cf_section_sub_find(maincs, "modules"), "radutmp");
+        cs = cf_section_sub_find(cf_section_find("modules"), "radutmp");
         if(!cs) {
                 fprintf(stderr, "%s: No configuration information in radutmp section of radiusd.conf!\n",
                         argv[0]);
@@ -222,9 +215,6 @@ int main(int argc, char **argv)
 
         cf_section_parse(cs, NULL, module_config);
 
-        /* Assign the correct path for the radutmp file */
-        radutmp_file = radutmpconfig.radutmp_fn;
-
 	printf("%s: zapping termserver %s, port %u",
 		progname, ip_hostname(buf, sizeof(buf), ip), nas_port);
 	if (user != NULL) 
@@ -255,13 +245,15 @@ static int getport(const char *name)
 	return ntohs(svp->s_port);
 }
 
-static const char *getlocalhostsecret(void)
+static const char *getsecret(uint32_t server)
 {
 	RADCLIENT *cl;
 
-	cl = client_find(htonl(INADDR_LOOPBACK));
+	cl = client_find(server);
 	if (cl == NULL) {
-		radlog(L_ERR|L_CONS, "No clients entry for localhost");
+		char buf[32];
+		radlog(L_ERR|L_CONS, "No clients entry for %s",
+		       ip_ntoa(buf,server));
 		exit(1);
 	}
 	return (const char *)cl->secret;
@@ -297,7 +289,7 @@ static int do_packet(int allports, uint32_t nasaddr, const struct radutmp *u)
 	struct timeval tv;
 	RADIUS_PACKET *req, *rep = NULL;
 	VALUE_PAIR *vp;
-	const char *secret=getlocalhostsecret();
+	const char *secret;
 
 	if ((req = rad_alloc(1)) == NULL) {
 		librad_perror("radzap");
@@ -319,6 +311,7 @@ static int do_packet(int allports, uint32_t nasaddr, const struct radutmp *u)
 	if(!req->dst_ipaddr) 
 		req->dst_ipaddr = 0x7f000001;
 	req->vps = NULL;
+	secret = getsecret(req->dst_ipaddr);
 
 	if(allports != 0) {
 		INTPAIR(PW_ACCT_STATUS_TYPE, PW_STATUS_ACCOUNTING_OFF);
@@ -333,7 +326,7 @@ static int do_packet(int allports, uint32_t nasaddr, const struct radutmp *u)
 		IPPAIR(PW_NAS_IP_ADDRESS, u->nas_address);
 		INTPAIR(PW_ACCT_DELAY_TIME, 0);
 		STRINGPAIR(PW_USER_NAME, login);
-		INTPAIR(PW_NAS_PORT_ID, u->nas_port);
+		INTPAIR(PW_NAS_PORT, u->nas_port);
 		STRINGPAIR(PW_ACCT_SESSION_ID, session_id);
 		if(u->proto=='P') {
 			INTPAIR(PW_SERVICE_TYPE, PW_FRAMED_USER);
