@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.49 2002/10/28 17:23:28 aland Exp $
+ * Version:	$Id: threads.c,v 1.63.2.2 2003/07/30 15:46:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,13 +39,14 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.49 2002/10/28 17:23:28 aland Exp $";
+"$Id: threads.c,v 1.63.2.2 2003/07/30 15:46:22 aland Exp $";
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
 
 #define THREAD_RUNNING		(1)
 #define THREAD_CANCELLED	(2)
+#define THREAD_EXITED		(3)
 
 /*
  *	Prototype to shut the compiler up.
@@ -68,14 +69,14 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun);
 typedef struct THREAD_HANDLE {
 	struct THREAD_HANDLE *prev;
 	struct THREAD_HANDLE *next;
-	pthread_t pthread_id;
-	int thread_num;
-	sem_t semaphore;
-	int status;
-	unsigned int request_count;
-	time_t timestamp;
-	REQUEST *request;
-	RAD_REQUEST_FUNP fun;
+	pthread_t            pthread_id;
+	int                  thread_num;
+	sem_t                semaphore;
+	int                  status;
+	unsigned int         request_count;
+	time_t               timestamp;
+	REQUEST              *request;
+	RAD_REQUEST_FUNP     fun;
 } THREAD_HANDLE;
 
 /*
@@ -86,9 +87,10 @@ typedef struct THREAD_HANDLE {
 typedef struct THREAD_POOL {
 	THREAD_HANDLE *head;
 	THREAD_HANDLE *tail;
+
+	THREAD_HANDLE *next_pick;
 	
 	int total_threads;
-	int active_threads;
 	int max_thread_num;
 	int start_threads;
 	int max_threads;
@@ -116,9 +118,13 @@ typedef struct rad_fork_t {
 	pid_t		child_pid;
 	sem_t	 	child_done;
 	int		status;	/* exit status of the child */
+	time_t		time_forked;
 } rad_fork_t;
 
-#define NUM_FORKERS (1024)
+/*
+ *  This MUST be a power of 2 for it to work properly!
+ */
+#define NUM_FORKERS (8192)
 static rad_fork_t forkers[NUM_FORKERS];
 
 /*
@@ -175,11 +181,22 @@ static void *request_handler_thread(void *arg)
 	 */
 	for (;;) {
 		/*
-		 *	Wait for the semaphore to be given to us.
+		 *	Wait to be signalled.
 		 */
 		DEBUG2("Thread %d waiting to be assigned a request",
-				self->thread_num);
-		if (sem_wait(&self->semaphore) < 0) {
+		       self->thread_num);
+	re_wait:
+		if (sem_wait(&self->semaphore) != 0) {
+			/*
+			 *	Interrupted system call.  Go back to
+			 *	waiting, but DON'T print out any more
+			 *	text.
+			 */
+			if (errno == EINTR) {
+				goto re_wait;
+			}
+			radlog(L_ERR, "Thread %d failed waiting for semaphore: %s: Exiting\n",
+			       self->thread_num, strerror(errno));
 			break;
 		}
 
@@ -192,7 +209,15 @@ static void *request_handler_thread(void *arg)
 					self->thread_num);
 			break;
 		}
-		
+
+		/*
+		 *	Stupid implementations of sem_wait return on
+		 *	signals, but don't return -1.
+		 */
+		if (!self->request) {
+			continue;
+		}
+
 		DEBUG2("Thread %d handling request %d, (%d handled so far)",
 				self->thread_num, self->request->number,
 				self->request_count);
@@ -201,22 +226,18 @@ static void *request_handler_thread(void *arg)
 		 *  This responds, and resets request->child_pid
 		 */
 		rad_respond(self->request, self->fun);
-		self->request = NULL;
 
-		/*
-		 *	The semaphore's value is zero, because we've
-		 *	locked it.  We now go back to the top of the loop,
-		 *	where we wait for it's value to become non-zero.
-		 */
+		self->fun = NULL;
+		self->request = NULL;
 	}
 
+	DEBUG2("Thread %d exiting...", self->thread_num);
+
 	/*
-	 *	This thread is exiting.  Delete any additional resources
-	 *	associated with it (semaphore, etc), and free the thread
-	 *	handle memory.
+	 *  Do this as the LAST thing before exiting.
 	 */
-	sem_destroy(&self->semaphore);
-	free(self);
+	self->status = THREAD_EXITED;
+
 	return NULL;
 }
 
@@ -237,6 +258,9 @@ static void delete_thread(THREAD_HANDLE *handle)
 	rad_assert(thread_pool.total_threads > 0);
 	thread_pool.total_threads--;
 
+	/*
+	 *	Remove the handle from the list.
+	 */
 	if (prev == NULL) {
 		rad_assert(thread_pool.head == handle);
 		thread_pool.head = next;
@@ -250,92 +274,29 @@ static void delete_thread(THREAD_HANDLE *handle)
 	} else {
 		next->prev = prev;
 	}
+
+	/*
+	 *	Ensure that deleted threads aren't picked.
+	 */
+	if (thread_pool.next_pick == handle) {
+		thread_pool.next_pick = next;
+	}
+
+	DEBUG2("Deleting thread %d", handle->thread_num);
+
+	/*
+	 *	This thread has exited.  Delete any additional
+	 *	resources associated with it (semaphore, etc).
+	 */
+	sem_destroy(&handle->semaphore);
+
+	/*
+	 *	Free the memory, now that we're sure the thread
+	 *	exited.
+	 */
+	free(handle);
 }
 
-/*
- *	Take a THREAD_HANDLE, and move it to the end of the thread pool.
- *
- *	This function is called ONLY from the main server thread.
- *	It's function is to keep the incoming requests rotating among
- *	the threads in the pool.
- */
-static void move2tail(THREAD_HANDLE *handle)
-{
-	THREAD_HANDLE *prev;
-	THREAD_HANDLE *next;
-
-	/*
-	 *	Empty list: add it to the head.
-	 */
-	if (thread_pool.head == NULL) {
-		rad_assert(thread_pool.tail == NULL);
-		rad_assert(thread_pool.total_threads == 1);
-
-		handle->prev = NULL;
-		handle->next = NULL;
-		thread_pool.head = handle;
-		thread_pool.tail = handle;
-		return;
-	}
-
-	/*
-	 *	It's already at the tail.  Stop.
-	 */
-	if (thread_pool.tail == handle) {
-		return;
-	}
-
-	rad_assert(thread_pool.total_threads >= 1);
-	prev = handle->prev;
-	next = handle->next;
-  
-	/*
-	 *	If the element is in the list, then delete it from where
-	 *	it is.
-	 */
-	if ((next != NULL) ||
-			(prev != NULL)) {
-		/*
-		 *	If it's already at the tail, exit immediately,
-		 *	there's no more work to do.
-		 */
-		if (next == NULL) {
-			rad_assert(thread_pool.tail == handle);
-			return;
-		}
-
-		/*
-		 *	Maybe it's at the head of the list?
-		 */
-		if (prev == NULL) {
-			rad_assert(thread_pool.head == handle);
-			thread_pool.head = next;
-			next->prev = NULL;
-
-			/*
-			 *	Nope, it's really in the middle.
-			 *	Unlink it, then.
-			 */
-		} else {
-			rad_assert(prev != NULL); /* be explicit about it. */
-			rad_assert(next != NULL); /* be explicit about it. */
-
-			prev->next = next;
-			next->prev = prev;
-		}
-	}
-
-	/*
-	 *	Finally, add it to the tail, and update the pointers.
-	 */
-	handle->next = NULL;
-	prev = thread_pool.tail;
-	rad_assert(prev->next == NULL);
-
-	thread_pool.tail = handle;
-	handle->prev = prev;
-	prev->next = handle;
-}
 
 /*
  *	Spawn a new thread, and place it in the thread pool.
@@ -372,12 +333,12 @@ static THREAD_HANDLE *spawn_thread(time_t now)
 
 	/*
 	 *	Initialize the semaphore to be for this process only,
-	 *	and to have the thread block until the server gives it
-	 *	the semaphore.
+	 *	and locked, so that the thread block until the server
+	 *	kicks it.
 	 */
 	rcode = sem_init(&handle->semaphore, 0, SEMAPHORE_LOCKED);
 	if (rcode != 0) {
-		radlog(L_ERR|L_CONS, "FATAL: Failed to allocate semaphore: %s",
+		radlog(L_ERR|L_CONS, "FATAL: Failed to initialize semaphore: %s",
 				strerror(errno));
 		exit(1);
 	}
@@ -422,9 +383,16 @@ static THREAD_HANDLE *spawn_thread(time_t now)
 			handle->thread_num, thread_pool.total_threads);
 
 	/*
-	 *	Move the thread handle to the tail of the thread pool list.
+	 *	Add the thread handle to the tail of the thread pool list.
 	 */
-	move2tail(handle);
+	if (thread_pool.tail) {
+		thread_pool.tail->next = handle;
+		handle->prev = thread_pool.tail;
+		thread_pool.tail = handle;
+	} else {
+		rad_assert(thread_pool.head == NULL);
+		thread_pool.head = thread_pool.tail = handle;
+	}
 
 	/*
 	 *	Update the time we last spawned a thread.
@@ -437,6 +405,24 @@ static THREAD_HANDLE *spawn_thread(time_t now)
 	return handle;
 }
 
+/*
+ *      Temporary function to prevent server from executing a SIGHUP
+ *      until all threads are finished handling requests.  This returns
+ *      the number of active threads to 'radiusd.c'.
+ */
+int total_active_threads(void) 
+{
+        int rcode = 0;
+	THREAD_HANDLE *handle;
+	
+	for (handle = thread_pool.head; handle != NULL; handle = handle->next){
+		if (handle->request != NULL) {
+			rcode ++;
+		}
+	}
+	return (rcode);
+}
+
 /*
  *	Allocate the thread pool, and seed it with an initial number
  *	of threads.
@@ -463,6 +449,7 @@ int thread_pool_init(void)
 		memset(&thread_pool, 0, sizeof(THREAD_POOL));
 		thread_pool.head = NULL;
 		thread_pool.tail = NULL;
+		thread_pool.next_pick = NULL;
 		thread_pool.total_threads = 0;
 		thread_pool.max_thread_num = 1;
 		thread_pool.cleanup_delay = 5;
@@ -516,25 +503,31 @@ int thread_pool_init(void)
  */
 int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 {
-	int active_threads;
 	THREAD_HANDLE *handle;
 	THREAD_HANDLE *found;
-	THREAD_HANDLE *next;
+	THREAD_HANDLE *start;
 
 	/*
 	 *	Loop over the active thread pool, looking for a
 	 *	waiting thread.
 	 */
 	found = NULL;
-	active_threads = 0;
-	for (handle = thread_pool.head; handle; handle = next) {
-		next = handle->next;
 
+	/*
+	 *	Minimize the time spent searching for an empty thread,
+	 *	by starting off from the last one we found.  This also
+	 *	helps to span requests across multiple threads.
+	 */
+	start = thread_pool.next_pick;
+	if (!start) start = thread_pool.head;
+	
+	handle = start;
+	while (handle) {
 		/*
 		 *	Ignore threads which aren't running.
 		 */
 		if (handle->status != THREAD_RUNNING) {
-			continue;
+			goto next_handle;
 		}
 
 		/*
@@ -544,12 +537,26 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 		 *	one locking it, waiting for us to unlock it.
 		 */
 		if (handle->request == NULL) {
-			if (found == NULL) {
-				found = handle;
-			}
-		} else {
-			active_threads++;
+			found = handle;
+			break;
 		}
+
+	next_handle:
+		/*
+		 *	There is a 'next', use it.
+		 *
+		 *	Otherwise, go back to the head of the list.
+		 */
+		if (handle->next) {
+			handle = handle->next;
+		} else {
+			handle = thread_pool.head;
+		}
+
+		/*
+		 *	Stop if we've looped back to the beginning.
+		 */
+		if (handle == start) break;
 	} /* loop over all of the threads */
 
 	/*
@@ -557,7 +564,7 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 *
 	 *	If we can't spawn a new one, complain, and exit.
 	 */
-	if (found == NULL) {
+	if (!found) {
 		found = spawn_thread(request->timestamp);
 		if (found == NULL) {
 			radlog(L_INFO, 
@@ -567,6 +574,12 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 		}
 	}
 
+	/*
+	 *	Remember where we left off, so that we can start the
+	 *	search from here the next time.
+	 */
+	thread_pool.next_pick = found->next;
+
 	/*
 	 *	OK, now 'handle' points to a waiting thread.  We move
 	 *	it to the tail of the thread pool, so that we can
@@ -577,14 +590,20 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 *	the request.
 	 */
 	DEBUG2("Thread %d assigned request %d", found->thread_num, request->number);
-	move2tail(found);
 	request->child_pid = found->pthread_id;
+
+	/*
+	 *	Tell the child thread what to do.
+	 */
 	found->request = request;
 	found->fun = fun;
 	found->request_count++;
 	thread_pool.request_count++;
+
+	/*
+	 * 	Now that the child has things to do, kick it.
+	 */
 	sem_post(&found->semaphore);
-	thread_pool.active_threads = active_threads;
 
 	return 0;
 }
@@ -603,12 +622,28 @@ int thread_pool_clean(time_t now)
 	int active_threads;
 
 	/*
-	 *	Loop over the thread pool, doing stuff.
+	 *	Loop over the thread pool, incrementing the 'active'
+	 *	counter if the thread is currently processing a request.
+	 *
+	 *	At this point, we also delete 'exited' threads.
 	 */
 	active_threads = 0;
-	for (handle = thread_pool.head; handle; handle = handle->next) {
+	for (handle = thread_pool.head; handle; handle = next) {
+		next = handle->next;
+
 		if (handle->request != NULL) {
 			active_threads++;
+			continue;
+		}
+
+		/*
+		 *	Handle is NULL, so it's not processing anything.
+		 *
+		 *	Maybe we've asked the thread to exit, and it
+		 *	has agreed.
+		 */
+		if (handle->status == THREAD_EXITED) {
+			delete_thread(handle);
 		}
 	}
 
@@ -688,15 +723,15 @@ int thread_pool_clean(time_t now)
 			 *	structure, without anyone else using it.
 			 */
 			if (handle->request == NULL) {
-				delete_thread(handle);
 				handle->status = THREAD_CANCELLED;
+
 				sem_post(&handle->semaphore);
 				spare--;
 				break;
 			}
 		}
 	}
-  
+
 	/*
 	 *	If the thread has handled too many requests, then make it
 	 *	exit.
@@ -706,11 +741,10 @@ int thread_pool_clean(time_t now)
 			next = handle->next;
 
 			/*
-			 *	Not handling a request, we can check it.
+			 *	Not handling a request, we can kill it.
 			 */
 			if ((handle->request == NULL) &&
 			    (handle->request_count > thread_pool.max_requests_per_thread)) {
-				delete_thread(handle);
 				handle->status = THREAD_CANCELLED;
 				sem_post(&handle->semaphore);
 			}
@@ -751,14 +785,20 @@ void rad_exec_init(void)
 	exec_initialized = TRUE;
 }
 
+/*
+ *	We use the PID number as a base for the array index, so that
+ *	we can quickly turn the PID into a free array entry, instead
+ *	of rooting blindly through the entire array.
+ */
+#define PID_2_ARRAY(pid) (((int) pid ) & (NUM_FORKERS - 1))
+
 /*
  *	Thread wrapper for fork().
  */
 pid_t rad_fork(int exec_wait)
 {
-	int i;
-	int found;
 	sigset_t set;
+	pid_t child_pid;	
 
 	/*
 	 *	The thread is NOT interested in waiting for the exit
@@ -772,41 +812,6 @@ pid_t rad_fork(int exec_wait)
 		return fork();
 	}
 
-	found = -1;
-
-	/*
-	 *	We may have multiple threads trying to find an empty
-	 *	position, so we lock the array until we've found an entry.
-	 */
-	pthread_mutex_lock(&fork_mutex);
-
-	/*
-	 *	FIXME: Keep a 'static' variable around of which entry
-	 *	was the last one freed.  This will reduce the search time
-	 *	in the critical section from O(N) to O(1).
-	 */
-	for (i = 0; i < NUM_FORKERS; i++) {
-		/*
-		 *	Empty entry, stop looking.
-		 */
-		if (forkers[i].thread_id == NO_SUCH_CHILD_PID) {
-			forkers[i].child_pid = -1; /* no possible match */
-			forkers[i].thread_id = pthread_self();
-			found = i;
-			break;
-		}
-	}
-	pthread_mutex_unlock(&fork_mutex);
-
-	/*
-	 *	Failed to find an open forking context.
-	 *
-	 *	Return an error.  (And fail to set 'errno')
-	 */
-	if (found < 0) {
-		return (pid_t) -1;
-	}
-
 	/*
 	 *	Block SIGCLHD until such time as we've saved the PID.
 	 *
@@ -820,20 +825,82 @@ pid_t rad_fork(int exec_wait)
 	/*
 	 *	Do the fork.
 	 */
-	forkers[found].child_pid = fork();
+	child_pid = fork();
 
 	/*
-	 *	Error: Clear the array entry.
+	 *	We managed to fork.  Let's see if we have a free
+	 *	array entry.
 	 */
-	if (forkers[found].child_pid < 0) {
-		forkers[found].thread_id = NO_SUCH_CHILD_PID;
+	if (child_pid > 0) { /* parent */
+		int i;
+		int found;
+		time_t now = time(NULL);
+
+		/*
+		 *	We store the information in the array
+		 *	indexed by PID.  This means that we have
+		 *	on average an O(1) lookup to find the element,
+		 *	instead of rooting through the entire array.
+		 */
+		i = PID_2_ARRAY(child_pid);
+		found = -1;
 		
-	} else if (forkers[found].child_pid > 0) {
+		/*
+		 *	We may have multiple threads trying to find an
+		 *	empty position, so we lock the array until
+		 *	we've found an entry.
+		 */
+		pthread_mutex_lock(&fork_mutex);
+		do {
+			if (forkers[i].thread_id == NO_SUCH_CHILD_PID) {
+				found = i;
+				break;
+			}
+
+			/*
+			 *	Clean up any stale forked sessions.
+			 *
+			 *	This sometimes happens, for crazy reasons.
+			 */
+			if ((now - forkers[i].time_forked) > 30) {
+				forkers[i].thread_id = NO_SUCH_CHILD_PID;
+
+				/*
+				 *	Grab the child's exit condition,
+				 *	just in case...
+				 */
+				waitpid(forkers[i].child_pid,
+					&forkers[i].status,
+					WNOHANG);
+				sem_destroy(&forkers[i].child_done);
+				found = i;
+				break;
+			}
+
+			/*
+			 *  Increment it, within the array.
+			 */
+			i++;
+			i &= (NUM_FORKERS - 1);
+		} while (i != PID_2_ARRAY(child_pid));
+		pthread_mutex_unlock(&fork_mutex);
+
+		/*
+		 *	Arg.  We did a fork, and there was nowhere to
+		 *	put the answer.
+		 */
+		if (found < 0) {
+			return (pid_t) -1;
+		}
+
 		/*
 		 *	In the parent, set the status, and create the
 		 *	semaphore.
 		 */
 		forkers[found].status = -1;
+		forkers[found].child_pid = child_pid;
+		forkers[i].thread_id = pthread_self();
+		forkers[i].time_forked = now;
 		sem_init(&forkers[found].child_done, 0, SEMAPHORE_LOCKED);
 	}
 
@@ -846,7 +913,7 @@ pid_t rad_fork(int exec_wait)
 	/*
 	 *	Return whatever we were told.
 	 */
-	return forkers[found].child_pid;
+	return child_pid;
 }
 
 /*
@@ -867,16 +934,26 @@ pid_t rad_waitpid(pid_t pid, int *status, int options)
 	}
 
 	/*
-	 *	Find the PID to wait for.
+	 *	Find the PID to wait for, starting at an index within
+	 *	the array.  This makes the lookups O(1) on average,
+	 *	instead of O(n), when the array is filling up.
 	 */
 	found = -1;
-	for (i = 0; i < NUM_FORKERS; i++) {
+	i = PID_2_ARRAY(pid);
+	do {
+		/*
+		 *	We were the ones who forked this specific
+		 *	child.
+		 */
 		if ((forkers[i].thread_id == self) &&
 		    (forkers[i].child_pid == pid)) {
 			found = i;
 			break;
 		}
-	}
+
+		i++;
+		i &= (NUM_FORKERS - 1);
+	} while (i != PID_2_ARRAY(pid));
 
 	/*
 	 *	No thread ID found: we're trying to wait for a child
@@ -896,7 +973,11 @@ pid_t rad_waitpid(pid_t pid, int *status, int options)
 			return 0; /* no child available */
 		}
 	} else {		/* wait forever */
-		sem_wait(&forkers[found].child_done);
+	re_wait:
+		rcode = sem_wait(&forkers[found].child_done);
+		if ((rcode != 0) && (errno == EINTR)) {
+			goto re_wait;
+		}
 	}
 
 	/*
@@ -929,11 +1010,21 @@ int rad_savepid(pid_t pid, int status)
 {
 	int i;
 
+	/*
+	 *	Find the PID to wait for, starting at an index within
+	 *	the array.  This makes the lookups O(1) on average,
+	 *	instead of O(n), when the array is filling up.
+	 */
+	i = PID_2_ARRAY(pid);
+
 	/*
 	 *	Do NOT lock the array, as nothing else sets the
 	 *	status and posts the semaphore.
 	 */
-	for (i = 0; i < NUM_FORKERS; i++) {
+	do {
+		/*
+		 *	Any thread can get the sigchild...
+		 */
 		if ((forkers[i].thread_id != NO_SUCH_CHILD_PID) &&
 		    (forkers[i].child_pid == pid)) {
 			/*
@@ -944,13 +1035,18 @@ int rad_savepid(pid_t pid, int status)
 			sem_post(&forkers[i].child_done);
 
 			/*
-			 *  If the child is more than 60 seconds out
-			 *  of date, then delete it.
+			 *	FIXME: If the child is more than 60
+			 *	seconds out of date, then delete it.
+			 *
+			 *	That is, we've forked, and the forker
+			 *	is waiting nearly forever
 			 */
-
 			return 0;		
 		}
-	}
+
+		i++;
+		i &= (NUM_FORKERS - 1);
+	} while (i != PID_2_ARRAY(pid));
 
 	return -1;
 }
