@@ -1,7 +1,7 @@
 /*
  * rlm_counter.c
  *
- * Version:  $Id: rlm_counter.c,v 1.32.2.1 2003/07/11 19:51:34 aland Exp $
+ * Version:  $Id: rlm_counter.c,v 1.32.2.2 2003/08/26 12:34:48 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
 
 #define UNIQUEID_MAX_LEN 32
 
-static const char rcsid[] = "$Id: rlm_counter.c,v 1.32.2.1 2003/07/11 19:51:34 aland Exp $";
+static const char rcsid[] = "$Id: rlm_counter.c,v 1.32.2.2 2003/08/26 12:34:48 phampson Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -541,11 +541,14 @@ static int counter_accounting(void *instance, REQUEST *request)
 	 * Check if we need to watch out for a specific service-type. If yes then check it
 	 */
 	if (data->service_type != NULL) {
-		if ((proto_vp = pairfind(request->packet->vps, PW_SERVICE_TYPE)) == NULL)
+		if ((proto_vp = pairfind(request->packet->vps, PW_SERVICE_TYPE)) == NULL){
+			DEBUG("rlm_counter: Could not find Service-Type attribute in the request. Returning NOOP.");
 			return RLM_MODULE_NOOP;
-		if (proto_vp->lvalue != data->service_val)
+		}
+		if (proto_vp->lvalue != data->service_val){
+			DEBUG("rlm_counter: This Service-Type is not allowed. Returning NOOP.");
 			return RLM_MODULE_NOOP;
-
+		}
 	}	
 	
 
@@ -555,7 +558,7 @@ static int counter_accounting(void *instance, REQUEST *request)
 	 */
 	key_vp = (data->key_attr == PW_USER_NAME) ? request->username : pairfind(request->packet->vps, data->key_attr);
 	if (key_vp == NULL){
-		DEBUG("rlm_counter: Could not find the key-attribute in the request.");
+		DEBUG("rlm_counter: Could not find the key-attribute in the request. Returning NOOP.");
 		return RLM_MODULE_NOOP;
 	}
 
