@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id: rlm_pap.c,v 1.9.2.1 2002/12/09 18:50:05 aland Exp $
+ * Version:  $Id: rlm_pap.c,v 1.13 2003/07/07 19:08:38 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 	free((char *)inst->scheme); \
 	free(inst)
 
-static const char rcsid[] = "$Id: rlm_pap.c,v 1.9.2.1 2002/12/09 18:50:05 aland Exp $";
+static const char rcsid[] = "$Id: rlm_pap.c,v 1.13 2003/07/07 19:08:38 aland Exp $";
 
 /*
  *      Define a structure for our module configuration.
@@ -58,9 +58,6 @@ static const char rcsid[] = "$Id: rlm_pap.c,v 1.9.2.1 2002/12/09 18:50:05 aland
 typedef struct rlm_pap_t {
         char *scheme;  /* password encryption scheme */
 	int sch;
-#if HAVE_PTHREAD_H
-	pthread_mutex_t mutex;	
-#endif
 } rlm_pap_t;
 
 /*
@@ -105,6 +102,10 @@ static int pap_instantiate(CONF_SECTION *conf, void **instance)
          *      Set up a storage area for instance data
          */
         inst = rad_malloc(sizeof(*inst));
+	if (!inst) {
+		return -1;
+	}
+	memset(inst, 0, sizeof(*inst));
 
         /*
          *      If the configuration parameters can't be parsed, then
@@ -124,9 +125,6 @@ static int pap_instantiate(CONF_SECTION *conf, void **instance)
 		inst->sch = PAP_ENC_CLEAR;
 	else if (strcasecmp(inst->scheme,"crypt") == 0){
 		inst->sch = PAP_ENC_CRYPT;
-#if HAVE_PTHREAD_H
-		pthread_mutex_init(&inst->mutex, NULL);
-#endif
 	}
 	else if (strcasecmp(inst->scheme,"md5") == 0)
 		inst->sch = PAP_ENC_MD5;
@@ -210,8 +208,8 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			break;
 		case PAP_ENC_CLEAR:
 			DEBUG("rlm_pap: Using clear text password.");
-			if (strncmp((char *) passwd_item->strvalue,
-				    (char *) request->password->strvalue, passwd_item->length) != 0){
+			if (strcmp((char *) passwd_item->strvalue,
+				   (char *) request->password->strvalue) != 0){
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: CLEAR TEXT password check failed");
 				module_fmsg_vp = pairmake("Module-Failure-Message",module_fmsg, T_OP_EQ);
@@ -221,25 +219,14 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			break;
 		case PAP_ENC_CRYPT:
 			DEBUG("rlm_pap: Using CRYPT encryption.");
-#if HAVE_PTHREAD_H
-			pthread_mutex_lock(&inst->mutex);
-#endif
-			if (strncmp((char *) passwd_item->strvalue,
-				crypt((char *) request->password->strvalue, (char *)passwd_item->strvalue),
-					passwd_item->length) != 0){
-#if HAVE_PTHREAD_H
-				pthread_mutex_unlock(&inst->mutex);
-#endif
+			if (lrad_crypt_check((char *) request->password->strvalue,
+								 (char *) passwd_item->strvalue) != 0) {
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: CRYPT password check failed");
 				module_fmsg_vp = pairmake("Module-Failure-Message",module_fmsg, T_OP_EQ);
 				pairadd(&request->packet->vps, module_fmsg_vp);
 				return RLM_MODULE_REJECT;
 			}
-#if HAVE_PTHREAD_H
-			else
-				pthread_mutex_unlock(&inst->mutex);
-#endif
 			break;
 		case PAP_ENC_MD5:
 			DEBUG("rlm_pap: Using MD5 encryption.");
@@ -257,7 +244,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			MD5Final(digest, &md5_context);
 			pap_hexify(buff,digest,16);
 			buff[32] = '\0';
-			if (strncmp((char *)passwd_item->strvalue, buff, passwd_item->length) != 0){
+			if (strcmp((char *)passwd_item->strvalue, buff) != 0){
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: MD5 password check failed");
 				module_fmsg_vp = pairmake("Module-Failure-Message",module_fmsg, T_OP_EQ);
@@ -282,7 +269,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			SHA1Final(digest,&sha1_context);
 			pap_hexify(buff,digest,20);
 			buff[40] = '\0';
-			if (strncmp((char *)passwd_item->strvalue, buff, passwd_item->length) != 0){
+			if (strcmp((char *)passwd_item->strvalue, buff) != 0){
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: SHA1 password check failed");
 				module_fmsg_vp = pairmake("Module-Failure-Message",module_fmsg, T_OP_EQ);
@@ -301,10 +288,6 @@ static int pap_detach(void *instance)
 {
 	rlm_pap_t *inst = (rlm_pap_t *) instance;
 
-#if HAVE_PTHREAD_H
-	if (inst->sch == PAP_ENC_CRYPT)
-		pthread_mutex_destroy(&inst->mutex);
-#endif
 	PAP_INST_FREE(inst);
 	return 0;
 }
